const { EmbedBuilder, PermissionsBitField, ActionRowBuilder, StringSelectMenuBuilder, Embed, ComponentType } = require("discord.js");
const { SlashCommandBuilder } = require("@discordjs/builders");
const config = require('../../config.json');
const fetch = require('node-fetch');
const getRandomInt = require('../functions/getRandomInt.js');
const fs = require('fs');

module.exports = {
  data: new SlashCommandBuilder()
    .setName("custommap")
    .setDescription("Generate custom maps")
    .addStringOption(Option => Option.setName('amount').setDescription("The amount of maps that you want to generate").setRequired(true))
    .addStringOption(Option => Option.setName('size').setDescription("What size you want the maps to be").setRequired(true))
    .addBooleanOption(Option => Option.setName('barren').setDescription("Do you want the map to be barren").setRequired(true))
    .addBooleanOption(Option => Option.setName('staging').setDescription("Do you want to generate on staging").setRequired(true)),

    run: async (client, interaction) => {
      
      let embed = new EmbedBuilder();
      let mapsArray = [];
      let theIndex = 0;
      let amountOfMaps = "";
      let mapSize = "";
      if(!config.RUSTMAPS_GENERATE_CUSTOM_MAPS.ENABLED) return interaction.editReply({ embeds: [embed.setDescription("This command is not enabled")] });
      if(config.RUSTMAPS_GENERATE_CUSTOM_MAPS.REQUIRE_ROLES && config.RUSTMAPS_GENERATE_CUSTOM_MAPS.REQUIRED_ROLES.find(x => interaction.member.roles.cache.has(x))) {
        interaction.editReply({ embeds: [embed.setDescription("You do not have permission to use this command.")] })
      } else {
        let amountOfMaps = interaction.options._hoistedOptions[0].value;
        let mapSize = interaction.options._hoistedOptions[1].value;
        let theConfigs = [];
        const theHeaders = { "Content-Type": "application/json", "X-API-Key": `${config.RUSTMAPS_API_KEY}` };
        fetch(`https://api.rustmaps.com/v4/maps/custom/saved-configs`, { method: 'get', headers: theHeaders }).then(res => res.json()).then(async response => {
          try {
            
            if(response.meta.status.toLowerCase() !== "success") return interaction.editReply("Could not load your saved configs on rustmaps");

            response.data.forEach((config, index) => {
              theConfigs.push({"label": config.name, "description": `Select ${config.name}`, "value": `${index}`});
            })
  
            let theConfigSelection = new ActionRowBuilder().addComponents( new StringSelectMenuBuilder().setCustomId("selectconfig").setPlaceholder("Select a config").addOptions(theConfigs));
            let interactionReply = await interaction.editReply({ embeds: [embed.setDescription("Please select a config to generate maps with")], components: [theConfigSelection] });
            await configSelection(interactionReply).then(async theConfig => {
              for(let times = 0; times < amountOfMaps; times++) {
                let theBody = {};
                theBody.customMapSettings = response.data[theConfig].settings;
                theBody.mapParameters = {
                  "seed": Math.floor(Math.random() * 2000000000),
                  "size": mapSize,
                  "barren": interaction.options._hoistedOptions[2].value,
                  "staging": interaction.options._hoistedOptions[3].value
               };
               let theSizeSeed = ``+"``"+`${theBody.mapParameters.seed}`+"``"+` / `+"``"+`${theBody.mapParameters.size}`+"``"+``;
                theBody = JSON.stringify(theBody);
                fetch(`https://api.rustmaps.com/v4/maps/custom`, { method: 'post', body: theBody, headers: theHeaders }).then(res => res.text()).then(rustMapsResponse => {
                  theIndex = theIndex + 1;
                  try {
                    rustMapsResponse = JSON.parse(rustMapsResponse);
    
                    if(rustMapsResponse.meta.statusCode !== 201) {
                      mapsArray.push("> ``" + theIndex + "``" + ` | ${rustMapsResponse.meta.errors[0]}`);
                    } else {
                      mapsArray.push("> ``" + theIndex + "``" + ` | **[${rustMapsResponse.data.mapId}](https://rustmaps.com/maps/${rustMapsResponse.data.mapId})** \n> (${theSizeSeed}) **${rustMapsResponse.data.state} (${rustMapsResponse.data.queuePosition})**`);
                    }
      
                    if(parseInt(theIndex) == parseInt(amountOfMaps)) {
                      if(parseInt(mapsArray.length) !== parseInt(amountOfMaps)) {
                        interaction.editReply({ components: [], embeds: [embed.setDescription("There was an error while generating your maps. Please check bot console.")] })
                      } else if(parseInt(mapsArray.length) == parseInt(amountOfMaps)) {
                        interaction.editReply({ components: [], ephemeral: false, embeds: [embed.setDescription(mapsArray.join("\n\n")).setAuthor({ name: `Generating ${amountOfMaps} map(s)`, iconURL: "https://cdn.discordapp.com/attachments/670451699063980083/936706897048920104/Icon_Primary.png" }).setFooter({ text: "Started generating" }).setTimestamp()] });
                        try {
                          interaction.user.send({ embeds: [embed.setDescription(mapsArray.join("\n\n")).setAuthor({ name: `Generating ${amountOfMaps} map(s)`, iconURL: "https://cdn.discordapp.com/attachments/670451699063980083/936706897048920104/Icon_Primary.png" }).setFooter({ text: "Started generating" }).setTimestamp()] })
                        } catch(err) {

                        }
                      }
                    }
      
                  } catch (err) {
                    console.log(err);
                  }
                });
              }
            }).catch(err => { interaction.editReply({ embeds: [embed.setDescription(err)], components: [] }) });
          } catch(err) {
            console.log(err);
          }
        });
      }
    }
 };

 function configSelection(interactionReply) {
  return new Promise((resolve, reject) => {
      const collector = interactionReply.createMessageComponentCollector({ componentType: ComponentType.StringSelect, time: 30000, max: 1 });
      collector.on('collect', async (i) => {
          i.deferUpdate();
          resolve(i.values[0]);
      });
  
      collector.on('end', collected => {
          if(collected.size < 1) {
              reject("Timed out");
          }
      });

  });
};
